=== modified file 'NEWS'
--- NEWS	2009-04-02 06:19:45 +0000
+++ NEWS	2009-04-02 16:35:02 +0000
@@ -38,7 +38,7 @@
   respectively, but have support for filtered views disabled.
   (Ian Clatworthy)
 
-* New ``guess-renames`` command recognizes renames after they occur.
+* New ``mv --auto`` option recognizes renames after they occur.
   (Aaron Bentley)
 
 * ``bzr log`` now supports filtering of multiple files and directories

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2009-03-31 11:33:02 +0000
+++ bzrlib/builtins.py	2009-04-01 21:59:09 +0000
@@ -722,14 +722,19 @@
     takes_args = ['names*']
     takes_options = [Option("after", help="Move only the bzr identifier"
         " of the file, because the file has already been moved."),
+        Option('auto', help='Automatically guess renames.'),
+        Option('dry-run', help='Avoid making changes when guessing renames.'),
         ]
     aliases = ['move', 'rename']
     encoding_type = 'replace'
 
-    def run(self, names_list, after=False):
+    def run(self, names_list, after=False, auto=False, dry_run=False):
+        if auto:
+            return self.run_auto(names_list, after, dry_run)
+        elif dry_run:
+            raise errors.BzrCommandError('--dry-run requires --auto.')
         if names_list is None:
             names_list = []
-
         if len(names_list) < 2:
             raise errors.BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list, canonicalize=False)
@@ -739,6 +744,20 @@
         finally:
             tree.unlock()
 
+    def run_auto(self, names_list, after, dry_run):
+        if names_list is not None and len(names_list) > 1:
+            raise errors.BzrCommandError('Only one path may be specified to'
+                                         ' --auto.')
+        if after:
+            raise errors.BzrCommandError('--after cannot be specified with'
+                                         ' --auto.')
+        work_tree, file_list = tree_files(names_list, default_branch='.')
+        work_tree.lock_write()
+        try:
+            rename_map.RenameMap.guess_renames(work_tree, dry_run)
+        finally:
+            work_tree.unlock()
+
     def _run(self, tree, names_list, rel_names, after):
         into_existing = osutils.isdir(names_list[-1])
         if into_existing and len(names_list) == 2:
@@ -5268,22 +5287,6 @@
             urlutils.unescape_for_display(to_branch.base, 'utf-8'))
 
 
-class cmd_guess_renames(Command):
-    """Guess which files have been have been renamed, based on their content.
-
-    Only versioned files which have been deleted are candidates for rename
-    detection, and renames to ignored files will not be detected.
-    """
-
-    def run(self):
-        work_tree, file_list = tree_files(None, default_branch='.')
-        work_tree.lock_write()
-        try:
-            rename_map.RenameMap.guess_renames(work_tree)
-        finally:
-            work_tree.unlock()
-
-
 class cmd_view(Command):
     """Manage filtered views.
 

=== modified file 'bzrlib/rename_map.py'
--- bzrlib/rename_map.py	2009-03-24 12:54:05 +0000
+++ bzrlib/rename_map.py	2009-03-24 15:47:32 +0000
@@ -20,6 +20,7 @@
 from bzrlib import (
     osutils,
     progress,
+    trace,
 )
 from bzrlib.ui import ui_factory
 
@@ -202,7 +203,7 @@
         return missing_files, missing_parents, candidate_files
 
     @classmethod
-    def guess_renames(klass, tree):
+    def guess_renames(klass, tree, dry_run=False):
         """Guess which files to rename, and perform the rename.
 
         We assume that unversioned files and missing files indicate that
@@ -235,12 +236,16 @@
                                                    missing_parents)
                 matches.update(parents_matches)
             pp.next_phase()
-            rn._update_tree(required_parents, matches)
+            delta = rn._make_inventory_delta(matches)
+            for old, new, file_id, entry in delta:
+                trace.note("%s => %s", old, new)
+            if not dry_run:
+                tree.add(required_parents)
+                tree.apply_inventory_delta(delta)
         finally:
             task.finished()
 
-    def _update_tree(self, required_parents, matches):
-        self.tree.add(required_parents)
+    def _make_inventory_delta(self, matches):
         delta = []
         file_id_matches = dict((f, p) for p, f in matches.items())
         for old_path, entry in self.tree.iter_entries_by_dir(matches.values()):
@@ -249,8 +254,10 @@
             parent_id = matches.get(parent_path)
             if parent_id is None:
                 parent_id = self.tree.path2id(parent_path)
+            if entry.name == new_name and entry.parent_id == parent_id:
+                continue
             new_entry = entry.copy()
             new_entry.parent_id = parent_id
             new_entry.name = new_name
             delta.append((old_path, new_path, new_entry.file_id, new_entry))
-        self.tree.apply_inventory_delta(delta)
+        return delta

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2009-03-24 12:15:01 +0000
+++ bzrlib/tests/blackbox/__init__.py	2009-04-01 20:30:51 +0000
@@ -59,7 +59,6 @@
                      'bzrlib.tests.blackbox.test_filesystem_cicp',
                      'bzrlib.tests.blackbox.test_filtered_view_ops',
                      'bzrlib.tests.blackbox.test_find_merge_base',
-                     'bzrlib.tests.blackbox.test_guess_renames',
                      'bzrlib.tests.blackbox.test_help',
                      'bzrlib.tests.blackbox.test_hooks',
                      'bzrlib.tests.blackbox.test_ignore',

=== removed file 'bzrlib/tests/blackbox/test_guess_renames.py'
--- bzrlib/tests/blackbox/test_guess_renames.py	2009-03-24 12:54:05 +0000
+++ bzrlib/tests/blackbox/test_guess_renames.py	1970-01-01 00:00:00 +0000
@@ -1,24 +0,0 @@
-# Copyright (C) 2008 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-
-from bzrlib import tests
-
-
-class TestGuessRenames(tests.TestCaseWithTransport):
-
-    def test_guess_renames(self):
-        self.run_bzr('guess-renames')

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2009-04-01 20:30:51 +0000
@@ -430,3 +430,58 @@
         self.assertNotInWorkingTree('c')
         self.failUnlessExists('d')
         self.assertInWorkingTree('d')
+
+    def make_abcd_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/c'])
+        tree.add(['a', 'c'])
+        tree.commit('record old names')
+        osutils.rename('tree/a', 'tree/b')
+        osutils.rename('tree/c', 'tree/d')
+        return tree
+
+    def test_mv_auto(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto', working_dir='tree')
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'a => b\nc => d\n')
+        tree = workingtree.WorkingTree.open('tree')
+        self.assertIsNot(None, tree.path2id('b'))
+        self.assertIsNot(None, tree.path2id('d'))
+
+    def test_mv_auto_one_path(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto tree')
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'a => b\nc => d\n')
+        tree = workingtree.WorkingTree.open('tree')
+        self.assertIsNot(None, tree.path2id('b'))
+        self.assertIsNot(None, tree.path2id('d'))
+
+    def test_mv_auto_two_paths(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto tree tree2', retcode=3)
+        self.assertEqual('bzr: ERROR: Only one path may be specified to'
+                         ' --auto.\n', err)
+
+    def test_mv_auto_dry_run(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto --dry-run', working_dir='tree')
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'a => b\nc => d\n')
+        tree = workingtree.WorkingTree.open('tree')
+        self.assertIsNot(None, tree.path2id('a'))
+        self.assertIsNot(None, tree.path2id('c'))
+
+    def test_mv_no_auto_dry_run(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv c d --dry-run',
+                                working_dir='tree', retcode=3)
+        self.assertEqual('bzr: ERROR: --dry-run requires --auto.\n', err)
+
+    def test_mv_auto_after(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto --after', working_dir='tree',
+                                retcode=3)
+        self.assertEqual('bzr: ERROR: --after cannot be specified with'
+                         ' --auto.\n', err)

=== modified file 'bzrlib/tests/test_rename_map.py'
--- bzrlib/tests/test_rename_map.py	2009-03-24 12:54:05 +0000
+++ bzrlib/tests/test_rename_map.py	2009-03-30 20:59:42 +0000
@@ -17,6 +17,7 @@
 
 import os
 
+from bzrlib import trace
 from bzrlib.rename_map import RenameMap
 from bzrlib.tests import TestCaseWithTransport
 
@@ -158,3 +159,43 @@
         os.rename('tree/foo', 'tree/baz')
         RenameMap.guess_renames(tree)
         self.assertEqual('baz/empty', tree.id2path('empty-id'))
+
+    def test_guess_renames_dry_run(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['tree/file'])
+        tree.add('file', 'file-id')
+        tree.commit('Added file')
+        os.rename('tree/file', 'tree/file2')
+        RenameMap.guess_renames(tree, dry_run=True)
+        self.assertEqual('file', tree.id2path('file-id'))
+
+    @staticmethod
+    def captureNotes(cmd, *args, **kwargs):
+        notes = []
+        def my_note(fmt, *args):
+            notes.append(fmt % args)
+        old_note = trace.note
+        trace.note = my_note
+        try:
+            result = cmd(*args, **kwargs)
+        finally:
+            trace.note = old_note
+        return notes, result
+
+    def test_guess_renames_output(self):
+        """guess_renames emits output whether dry_run is True or False."""
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['tree/file'])
+        tree.add('file', 'file-id')
+        tree.commit('Added file')
+        os.rename('tree/file', 'tree/file2')
+        notes = self.captureNotes(RenameMap.guess_renames, tree,
+                                  dry_run=True)[0]
+        self.assertEqual('file => file2', ''.join(notes))
+        notes = self.captureNotes(RenameMap.guess_renames, tree,
+                                  dry_run=False)[0]
+        self.assertEqual('file => file2', ''.join(notes))

